//
//	"$Id: Conteiner.h The Dec 18 2006 Dmitry Perov  $"
//
//	Conteiner for the Fast Light Tool Kit (FLTK2).
//
//   Copyright 1998-2006 by Bill Spitzak and others.
//
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//	Please report all bugs and problems to "gentooshnik@rambler.ru"
//

#ifndef Conteiner_h
#define Conteiner_h 1

#include <sstream>
#include <map>


typedef std::multimap<int, std::string> StringMap;

class Conteiner {

public:

Conteiner();
~Conteiner();

/**
	Вставка элементов
*/

void insert(const char* data);
void insert(const char data);
void insert(const int data);
void insert(const std::string data);

/**
	Проверка на существование Элементов со значением data
*/

bool isExists(const char data);
bool isExists(const char* data);
bool isExists(const int data);
bool isExists(const std::string data);

/**
	Вставка элементов, если индекс существует то элемент = data, если нет то
	данные вставляются в конец
*/

void insert(const char data, const int index);
void insert(const char* data, const int index);
void insert(const int data, const int index);
void insert(const std::string data, const int index);

/**
	Добавление данных к элементу с индексом index, если индекс существует то элемент += data,
	если нет то данные вставляются в конец
*/

void append(const char data, const int index);
void append(const char* data, const int index);
void append(const int, const int index);
void append(const std::string data, const int index);

/**
	Возвращает индекс Элемента со значением data
*/

const int index(const char data);
const int index(const char *data);
const int index(const int data);
const int index(const std::string data);

/**
	Удаление элемента с индексом index
*/

bool remove_index(const int index);

/**
	Удаление элементов если data == Элемент.Данные()
*/

bool remove(const char data);
bool remove(const char *data);
bool remove(const int data);
bool remove(const std::string data);

/**
	Возвращает количество элементов со значением data
*/

const int num(const char data);
const int num(const char *data);
const int num(const int data);
const int num(const std::string data);

/**
	Удаление двойных значений в Контейнере
*/

void delete_doubles();

/**
	Проверяет/Устанавливает добавление двойных значений
*/

bool double_values();
void double_values(bool d);

/**
	Очищает Контейнер
*/

void clear();

/**
	Очищает Элемент с индексом index
*/

void clear(const int index);

/**
	Возвращает значение следуйщего/предыдущего Элемента, позиция обновляется
*/

const char* next();
const char* prev();
const std::string next_string();
const std::string prev_string();

/**
	Устанавливает/Возвращает позицию в Контейнере
*/

void set_position(const int index);
const int position();

/**
	Проверка на позицию в Конце/Начале
*/

bool isStart(const int index);
bool isEnd(const int index);

/**
	Возвращает позицию в Конце/Начале
*/

const int Start();
const int End();

/**
	Перемещает Элемент с индексом index в начало/конец Контейнера
*/

bool ToStart(const int index);
bool ToEnd(const int index);

/**
	Меняет местами Элементы с индексами index_a и index_b
*/

bool swap(const int index_a, const int index_b);

/**
	Возвращает Элемент с индексом index
*/

std::string element(const int index);

/**
	Проверка на существование Элемента с индексом index
*/

bool isIndex(const int index);

/**
	Проверка является ли элемент с индексом index числовым
*/

bool isInt(const int index);

/**
	Проверка на нулевой размер элемента с индексом index
*/

bool isEmpty(const int index);

/**
	Проверка на нулевой размер Контейнера
*/

bool isEmpty();

/**
	Возвращает значения элементов
*/

const char* value();
const char* value(const int index);
const std::string value_string(const int index);
const int value_int(const int index);

/**
	Возвращает кол-во символов в элементе с индексом index [ поддержка UTF-8 ]
*/

const int lenght(const int index);

/**
	Возвращает/Устанавливает размер, резервирующийся при добавлении элементов
*/

const int size();
void set_size(const int sz);

/**
	Размер Контейнера, или количество элементов в нём
*/

const int count();

/**
	Стырил из FLTK2 =)
*/

const int utf8len(const char* src);

/**
	Возвращает char* из int
	Где то на форуме подсматрел =)
*/

const char* itoa(const int in);

/**
	Из string в ?
	Где то на форуме подсматрел =)
*/

template<typename T> bool FromString(const std::string& from, T& to);

protected:

int _size;
int _position;
bool double_v;
StringMap *conteiner;

};

#endif

//
// End of  "$Id: Conteiner.h The Dec 18 2006 Dmitry Perov  $".
//
