//
//	"$Id: InputBrowserTmp.h The Dec 27 2006 Dmitry Perov  $"
//
//	InputBrowserTmp for the Fast Light Tool Kit (FLTK2).
//
//   Copyright 1998-2006 by Bill Spitzak and others.
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//	Please report all bugs and problems to "gentooshnik@rambler.ru"
//

#ifndef fltk_InputBrowserTmp_h
#define fltk_InputBrowserTmp_h

#include <fltk/Menu.h>
#include <fltk/Input.h>
#include <fltk/Button.h>
#include <fltk/Widget.h>

namespace fltk {

class FL_API InputTmp : public Input {
  public:

    InputTmp(int x, int y, int w, int h);
    int handle(int event);
    void update_cb();

    /** set widget/button for inserting images from selected item in menu */
    void set_button(Button* b){ img2but = b; }
    void set_widget(Widget* b){ img2w = b; }

    /** maximum items in menu [ default 100 ] */
    void max_items( const int n ){ max_item = n; }
    const int max_items(){ return max_item; }

    /** for inserting items with ReturnKey | KeypadEnter [ default true] */
    bool inserts(){ return insetr_items; }
    void inserts(bool b){ insetr_items = b; }

private:

    Button* img2but;
    Widget* img2w;

protected:

    int max_item;
    bool insetr_items;

};


class ComboBrowser;
class ComboWindow;

class FL_API InputBrowserTmp : public Menu {

public:

  enum { // values for type()
    NORMAL = 0,
    NONEDITABLE = 1,
    INDENTED = 2,
    NONEDITABLE_INDENTED = 3
  };

  InputBrowserTmp(int,int,int,int,const char * = 0);
  ~InputBrowserTmp();
  static NamedStyle* default_style;
  InputTmp m_input;
  void popup();
  void hide_popup();
  virtual int popup(int x, int y, int w, int h) { InputBrowserTmp::popup(); return Menu::popup(Rectangle(x,y,w,h)); }

  virtual int handle(int);

  Widget* item() const ;
  Widget* item(Widget* v) const ;

  void maxw(int i) { maxw_ = i; }
  void maxh(int i) { maxh_ = i; }
  int maxw() { return maxw_; }
  int maxh() { return maxh_; }

  void text(const char *v) { m_input.text(v); }
  const char *text() const { return m_input.text(); }

	/** for manip style menu */
  void mw_color(const Color c);
  void mw_text_color(const Color c);
  void mw_selection_color(const Color c);
  void mw_selection_textcolor(const Color c);
  void mw_textsize(const int sz);
  void mw_font(Font *f);
  void mw_box_style(Box *b);

  Color color_mw;
  Color text_color_mw;
  Color selection_color_mw;
  Color selection_textcolor_mw;
  int textsize_mw;
  Font *font_mw;
  Box *box_mw;

	/** for manip style scrollbars in menu */
  void scr_box(Box *b);
  void scr_bbox(Box *b);
  void scr_color(Color c);
  void scr_bcolor(Color c);
  void scr_hig_color(Color c);

  Box *s_box;
  Box *s_bbox;
  Color s_color;
  Color s_bcolor;
  Color hig_color;

protected:

  virtual void draw();
  static void input_cb(InputTmp *w, InputBrowserTmp *ib);


  ComboWindow *win;
  ComboBrowser *list;

  friend class ComboWindow;
  friend class ComboBrowser;

  int minw_, minh_, maxw_, maxh_;
  int over_now, over_last;
};

}

#endif

//
// End of "$Id: InputBrowserTmp.h The Dec 27 2006 Dmitry Perov  $" .
//
