//
//	"$Id: CoolInput.cxx The Dec 27 2006 Dmitry Perov  $"
//
//	CoolInput for the Fast Light Tool Kit (FLTK2).
//
//   Copyright 1998-2006 by Bill Spitzak and others.
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//	Please report all bugs and problems to "gentooshnik@rambler.ru"
//


#include <fltk/CoolInput.h>
#include "CoolInput_images.h"
#include "CoolInput_lang.h"
#include <fltk/events.h>
#include <fltk/Style.h>
#include <fltk/PopupMenu.h>
#include <fltk/Item.h>
#include <fltk/Divider.h>
#include <fltk/Group.h>
#include <fltk/xpmImage.h>


using namespace fltk;


static PopupMenu *pm;
static Item *il_cut;
static Item *il_copy;
static Item *il_paste;
static Item *il_select;
static Item *il_unselect;
static Item *il_cleanse;
static xpmImage *img_cut;
static xpmImage *img_copy;
static xpmImage *img_paste;
static Box *bs;
static CoolInput * This;


CoolInput::CoolInput(int x, int y, int w, int h, const char* label, const int lang)
: Input(x,y,w,h,label), cursor(CURSOR_INSERT) { _Lang = lang; This = this; }

CoolInput::~CoolInput(){ delete pm; delete il_cut; delete il_copy; delete il_paste; delete il_select; delete il_unselect;
delete il_cleanse; delete img_cut; delete img_copy; delete img_paste; delete font_popup; delete bs; delete [] *PLang; delete This; delete this; }

//void CoolInput::style_fltk(){ style_off = true; }
void CoolInput::set_color_popup(const Color c){ color_popup = c; }
void CoolInput::set_text_color_popup(const Color c){ text_color_popup = c; }
void CoolInput::set_selection_color_popup(const Color c){ selection_color_popup = c; }
void CoolInput::set_selection_textcolor_popup(const Color c){ selection_textcolor_popup = c; }
void CoolInput::set_textsize_popup(const int sz){ textsize_popup = sz; }
void CoolInput::set_font_popup(Font *f){ font_popup = f; }
void CoolInput::set_box_style_popup(Box *b){ bs = b; }
void CoolInput::_paste(int, CoolInput* c){ fltk::paste(*c, 1); }
void CoolInput::set_lang_popup(const int lng){ _Lang = lng; }
int CoolInput::lang_popup() const { return _Lang; }

static void cut_cb(fltk::CoolInput*, void* v){

CoolInput* c = (CoolInput*)v;
if(c->type() == 3){ return; }
c->copy();
c->cut(c->position(), c->mark());

}

static void copy_cb(fltk::CoolInput*, void* v){

CoolInput* c = (CoolInput*)v;
if(c->type() == 3){ return; }
c->copy();

}

static void paste_cb(fltk::CoolInput*, void* v){
CoolInput* c = (CoolInput*)v;

int s = c->position();
int e = c->mark();

if(s != e){
if(s > e){ s = c->mark(); e = c->position(); }
if(c->type() == 3){ e = s; }
c->_paste((int)e-s, c); }

else{ c->_paste(0, c); }

}

static void select_cb(fltk::CoolInput*, void* v){

CoolInput* c = (CoolInput*)v;
c->position(0,c->size());

}

static void unselect_cb(fltk::CoolInput*, void* v){
CoolInput* c = (CoolInput*)v;

int s = c->position();
int e = c->mark();
if(s < e){ s = e; }
c->position(s);

}

static void cleanse_cb(fltk::CoolInput*, void* v){
CoolInput* c = (CoolInput*)v;

int s = c->position();
int e = c->mark();

if(s != e){
if(s > e){ s = c->mark(); e = c->position(); }
c->replace(s, e, 0, 0);
}

else{ c->replace(0, c->size(), 0, 0); }

}


static void init_menu(){

int s = This->position();
int e = This->mark();
if(s > e){ s = e; e = This->position(); }

if(s == e){ il_cut->deactivate(); il_copy->deactivate(); il_unselect->deactivate(); }
if(This->size() <= 0){ il_select->deactivate(); il_cleanse->deactivate(); }
if(s == 0 && e == This->size()){ il_select->deactivate(); }
if(This->type() == 3){ il_cut->deactivate(); il_copy->deactivate(); }
/*  FIXME: If Buffer for fltk::paste == NULL : deactivate il_paste : edit next line */
// if( ... ){ il_paste->deactivate(); }

}

void CoolInput::init_style(){

if(bs || !style_off){

if(bs){ il_cut->box(bs); il_copy->box(bs); il_paste->box(bs);
il_select->box(bs); il_unselect->box(bs); il_cleanse->box(bs); }

else if(!style_off){ il_cut->box(BORDER_BOX); il_copy->box(BORDER_BOX); il_paste->box(BORDER_BOX);
il_select->box(BORDER_BOX); il_unselect->box(BORDER_BOX); il_cleanse->box(BORDER_BOX); }

}

if(style_off && !font_popup && !textsize_popup && !color_popup
 && !text_color_popup && !selection_color_popup
 && !selection_textcolor_popup){ return; }

if(font_popup){ pm->textfont(font_popup); }
else if(!style_off){ pm->textfont(TIMES); }

if(textsize_popup){ pm->textsize(textsize_popup); }
else if(!style_off){ pm->textsize(14); }

if(color_popup){ pm->color(color_popup); }
else if(!style_off){ pm->color((Color)0xff); }

if(text_color_popup){ pm->textcolor(text_color_popup); }
else if(!style_off){ pm->textcolor((Color)0x38); }

if(selection_color_popup){ pm->selection_color(selection_color_popup); }
else if(!style_off){ pm->selection_color((Color)0x526ba900); }

if(selection_textcolor_popup){ pm->selection_textcolor(selection_textcolor_popup); }
else if(!style_off){ pm->selection_textcolor((Color)0xff); }

}

int CoolInput::handle(int event) {

switch (event) {

case PUSH:

if(fltk::event_button() == 3 && take_focus()){

Input::cursor(CURSOR_DEFAULT);

img_cut = new fltk::xpmImage(cut_xpm);
img_copy = new fltk::xpmImage(copy_xpm);
img_paste = new fltk::xpmImage(paste_xpm);

PopupMenu *pop = pm = new PopupMenu(fltk::event_x_root(), fltk::event_y_root(), 140, 1);
pop->begin();

Item* item_cut = il_cut = new Item(PLang[_Lang][0], fltk::CTRL + 'x', (fltk::Callback *)cut_cb, this);
item_cut->image(img_cut);

Item* item_copy = il_copy = new Item(PLang[_Lang][1], fltk::CTRL + 'c', (fltk::Callback *)copy_cb, this);
item_copy->image(img_copy);

Item* item_paste = il_paste = new Item(PLang[_Lang][2], fltk::CTRL + 'v', (fltk::Callback *)paste_cb, this);
item_paste->image(img_paste);

new Divider();

Item* item_select = il_select = new Item(PLang[_Lang][3], 0, (fltk::Callback *)select_cb, this);

Item* item_unselect = il_unselect = new Item(PLang[_Lang][4], 0, (fltk::Callback *)unselect_cb, this);

Item* item_cleanse = il_cleanse = new Item(PLang[_Lang][5], 0, (fltk::Callback *)cleanse_cb, this);

pop->end();

pop->type(PopupMenu::POPUP3);
init_menu();
init_style();
pop->popup();

return 0; }

else if(active_r()){ Input::cursor(cursor); }

case ENTER: if(active_r()){ Input::cursor(cursor); }

}

return Input::handle(event); }

//
// End of  "$Id: CoolInput.cxx The Dec 27 2006 Dmitry Perov  $".
//
