//
//	"$Id: InputBrowserTmp.cxx The Dec 27 2006 Dmitry Perov  $"
//
//	InputBrowserTmp for the Fast Light Tool Kit (FLTK2).
//
//   Copyright 1998-2006 by Bill Spitzak and others.
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//	Please report all bugs and problems to "gentooshnik@rambler.ru"
//


#include <fltk/InputBrowserTmp.h>
#include <fltk/MenuWindow.h>
#include <fltk/Browser.h>
#include <fltk/Monitor.h>
#include <fltk/events.h>
#include <fltk/Item.h>
#include <fltk/damage.h>
#include <fltk/Box.h>
#include <fltk/draw.h>
#include <stdio.h>
#include <string.h>
#include <fltk/Style.h>
#include <fltk/utf.h>
#include <fltk/Symbol.h>
#include <fltk/Box.h>

using namespace fltk;

namespace fltk {

class ComboWindow : public MenuWindow {
  public:
    int handle(int);
    ComboWindow(int x, int y, int w, int h) : MenuWindow(x, y, w, h) { ; }
};

}


static const int lenght(const char * src){

int len = utf8test(src, strlen(src));

if(len == 0){ return strlen(src); }
else{ return len; }


}


static InputTmp *ibinput;

static NamedStyle style("InputBrowserTmp", 0, &InputBrowserTmp::default_style);
NamedStyle* InputBrowserTmp::default_style = &::style;

InputBrowserTmp::InputBrowserTmp(int x, int y, int w, int h, const char *l)
  : Menu(x, y, w, h, l),
    m_input(x, y, w, h)
{
  align(ALIGN_LEFT);
  style(default_style);

  if (m_input.parent()) m_input.parent()->remove(m_input);
  m_input.parent(this);
  m_input.callback((Callback_p)input_cb, this);
  m_input.when(fltk::WHEN_ENTER_KEY_CHANGED | fltk::WHEN_RELEASE); // FL_WHEN_CHANGED FL_WHEN_ENTER_KEY_ALWAYS

  over_now = false; over_last = true;

  minh_ = 10;
  maxw_ = 600;
  maxh_ = 320;

  win = 0; list = 0;

  ibinput = &m_input;
}

void InputBrowserTmp::input_cb(InputTmp *w, InputBrowserTmp *ib)
{
  ib->do_callback();
}


// these are only used when in grabbed state so only one exists at once
static MenuWindow *mw;
static InputBrowserTmp *ib;
static Browser *browser;

InputTmp::InputTmp(int x, int y, int w, int h) : Input(x, y, w, h){ max_item = 100; insetr_items = true; }

int InputTmp::handle(int event){

switch(event){

case KEY:

if(event_key() == fltk::ReturnKey || event_key() == fltk::KeypadEnter){

update_cb();

return 1;
 }
}
Input::handle(event); }


void InputTmp::update_cb(){

/**
FIXME: if ( !ib ) then initial and drawing ib, for inserting items
with ReturnKey | KeypadEnter in menu.

IF not open menu at start up program, then values input with
ReturnKey | KeypadEnter not insert to menu from input.

*/

const int len = lenght(text());

if(len > 0 && ib  && insetr_items){

Widget* w = ib->find(text());
const Symbol* s;
if(w){ s = w->image(); }

/**
		UP selected item

FIXME: necessity disable handle in menu for parsing first
simbols '/' , then correct remove values as "Program Files/fltk2".

IF text()[0] != '/' and exists '/' in text(), then not remove exists
text() in menu.

*/
ib->remove(text());
ib->insert(0, text());

	/** insert old image isExists */
if(w && s){
ib->find(text())->image(s);

if(img2but){ img2but->image(s); img2but->redraw(); }
if(img2w){ img2w->image(s); img2w->redraw(); }
}

else{
if(img2but){ img2but->image(0); img2but->redraw(); }
if(img2w){ img2w->image(0); img2w->redraw(); }
 }
}

	/** sizes menu */
if(ib){
int num = ib->children();
if(num  > max_item){
while(1){
if(ib->children() == max_item) break;
ib->remove(num);
--num; }
 }
}

	/** set position to end */
position(size()); }



int ComboWindow::handle(int event) {



  switch (event) {
  case MOVE:
  case DRAG:
  case RELEASE:
  case KEY:
  return browser->handle(event);
  }
  return MenuWindow::handle(event);
}


// Destructor needs to be after declaration of ComboWindow win
InputBrowserTmp::~InputBrowserTmp()
{
    m_input.parent(0);
    if(win) delete win;
}

namespace fltk {

class FL_API ComboBrowser : public Browser {
  public:
    int handle(int);
    ComboBrowser(int x, int y, int w, int h);
    static void browser_cb(Widget *w, void *data);
};

}

extern void browser_glyph(int glyph, const Rectangle&);

ComboBrowser::ComboBrowser(int x, int y, int w, int h)
: Browser(x, y, w, h, 0) {}


int ComboBrowser::handle(int event) {

  if(event_key()==DownKey && (!item() || children()==1)) {
    item(child(0));
    fltk::focus(item());
  }

  if (event == KEY && event_state(ACCELERATOR)) {
      return Browser::handle(event); // delegate to the Menu handler
  } else if ((event==SHORTCUT||event==KEY) && !(ib->type()&InputBrowserTmp::NONEDITABLE)) {
    if ((event_key()!=LeftAccKey) &&
	(event_key()!=RightAccKey) &&
        (event_key()!=EscapeKey) &&
	(event_key()!=UpKey) &&
	(event_key()!=DownKey) && 
	(event_key()!=ReturnKey && !item()) )
      // give a chance to the browser to handle the menu shortcuts
      return ibinput->handle(KEY);
  }

  static bool was_wheel=false;
  if(was_wheel) {
    was_wheel=false;
    return 1;
  }

  switch (event) {
  case MOUSEWHEEL: {
    was_wheel=true;
    break;
  }




  case SHORTCUT:
    if(event_key() == EscapeKey) {
      ib->hide_popup();
      return 1;
    }
    break;

  case PUSH: {
    if (!event_inside(Rectangle(0, 0, w(), h()))) {
      ib->hide_popup();
      // Rectangle below is InputBrowserTmp area
      if (event_inside(Rectangle(0, -ib->h(), ib->w(), 0))) ib->send(PUSH);

      return 1;
    }
    break;

  }

  case MOVE:
    event = DRAG;

  case RELEASE:
  case DRAG:
    // this causes a drag-in to the widget to work:
    if (event_inside(Rectangle(0, 0, w(), h()))) fltk::pushed(this);
    else {
      fltk::pushed(this);
      return 0;
    }

  default:
    break;
  }

  return Browser::handle(event);
}

void ComboBrowser::browser_cb(Widget*, void*) {
  // we get callbacks for all keys?

if (event() != KEY && event() != RELEASE) return;
  if (event() == KEY
      && event_key() != ReturnKey
      && event_key() != KeypadEnter
      && event_key() != ' ')
    return;
  Widget *item = browser->item();

  if (!item) return;
  if (item->is_group()) return; // can't select a group!

  ib->item(item);
  ib->text(item->label());
  ib->redraw(DAMAGE_VALUE);
  ib->hide_popup();
  ib->do_callback();
	/** callback InputTmp */
  ibinput->update_cb();
}

// CET - FIXME - this doesn't seem to be working
// Use this to copy all the items out of one group into another:
class Share_List : public List {
public:
  Menu* other;
  int children(const Menu*, const int* indexes, int level) {
    return other->children(indexes, level);
  }
  Widget* child(const Menu*, const int* indexes, int level) {
    return other->child(indexes, level);
  }
  void flags_changed(const Menu*, Widget* widget) {
    other->list()->flags_changed(other,widget);
  }
} share_list; // only one instance of this.

/** set styles for menu & scroll */

void InputBrowserTmp::mw_color(const Color c){ color_mw = c; }
void InputBrowserTmp::mw_text_color(const Color c){ text_color_mw = c; }
void InputBrowserTmp::mw_selection_color(const Color c){ selection_color_mw = c; }
void InputBrowserTmp::mw_selection_textcolor(const Color c){ selection_textcolor_mw = c; }
void InputBrowserTmp::mw_textsize(const int sz){ textsize_mw = sz; }
void InputBrowserTmp::mw_font(Font *f){ font_mw = f; }
void InputBrowserTmp::mw_box_style(Box *b){ box_mw = b; }


void InputBrowserTmp::scr_box(Box *b){ s_box = b; }
void InputBrowserTmp::scr_bbox(Box *b){ s_bbox = b; }
void InputBrowserTmp::scr_color(Color c){ s_color = c; }
void InputBrowserTmp::scr_bcolor(Color c){ s_bcolor = c; }
void InputBrowserTmp::scr_hig_color(Color c){ hig_color = c; }

int InputBrowserTmp::handle(int event) {
  int TX, TY = 0, TW, TH = h();
  if(type()&NONEDITABLE) {
      TX = 0;
      TW = w();
  } else {
      TX = m_input.x()+m_input.w();
      TW = w()-(m_input.x()+m_input.w());
  }
  if (event_inside(Rectangle(TX,TY,TW,TH)))
      over_now = true;
  else
      over_now = false;
  if((over_now != over_last) && highlight_color())
      redraw_highlight();
  //if (event == ENTER || event == LEAVE) redraw_highlight();

  if (event == FOCUS) fltk::focus(m_input);

  // Scroll using arrow keys
  if ((event == KEY) && (event_key() == UpKey || event_key() == DownKey )) {
    if (!win || !win->visible())
	popup();
//    else
    return win->handle(event);
  } else if (event == KEY && event_state(ACCELERATOR) ) {
      return list ? list->handle(event) : Menu::handle(event); // delegate to the Menu handler
  // all other keys should be sent to Input
  } else if ((event_inside(m_input) || event == KEY)
    && !(type()&NONEDITABLE) && !pushed() && event != MOUSEWHEEL) {
    if (event == PUSH) { fltk::pushed(m_input); fltk::focus(m_input); }
    return m_input.handle(event); // if this doesn't work, try send(e)
  }


  switch (event) {

    // Mouse wheel support
    case MOUSEWHEEL: {

      // prevent double events
      static bool was_wheel=false;
      if(was_wheel) {
	  was_wheel=false;
	  return 1;
      } else {
	  was_wheel=true;
      }

      // horizontal wheel
      if (event_dy() == 0)
	  break;

      // find next item in the list and replace the current
      int found=-1;
      for (int i=0; i<children(); i++) {
	  Widget* w=child(i);
	  if (!strncmp(text(), w->label(), size())) {
	      found=i; break;
	  }
      }
      if (event_dy() < 0) {
	  if (found==-1 || found==children()-1)
	      found=0;
	  else
	      found++;
      } else if (event_dy() > 0) {
	  if (found==-1 || found==0)
	      found=children()-1;
	  else
	      found--;
      }
	  m_input.text(child(found)->label());
      break;
    }

    case PUSH: {
      if (!win || !win->visible())
	  popup();
      else
	  hide_popup();
      return 1;
    }

    case FOCUS:
    case UNFOCUS:
      if (type()&NONEDITABLE) break;
      return m_input.handle(event);


 case ENTER: case MOVE: return 1;
  }
  return 0;
}

void InputBrowserTmp::draw() {
  drawstyle(style(),flags());
  minw_ = w();
  Rectangle r(w(),h()); box()->inset(r);
  int W1 = r.h()*4/5;
  if (damage()&(DAMAGE_ALL|DAMAGE_CHILD)) {
    if (damage()&DAMAGE_ALL) draw_frame();
    m_input.resize(r.x(), r.y(), r.w()-W1, r.h());
    m_input.set_damage(DAMAGE_ALL);
    m_input.copy_style(style()); // force it to use this style
    m_input.box(FLAT_BOX);

    // fix for relative coordinates
    push_matrix();
    translate(r.x(),r.y());

    m_input.draw();

    pop_matrix();
    m_input.set_damage(0);
  }
  if (damage()&(DAMAGE_ALL|DAMAGE_VALUE|DAMAGE_HIGHLIGHT)) {
    Flags f = flags() & ~FOCUSED | OUTPUT;
    if (over_now) f |= HIGHLIGHT;
    drawstyle(style(),f);

    // draw the little mark at the right:
    r.x(r.x()+r.w()-W1); r.w(W1);
    draw_glyph(ALIGN_BOTTOM|ALIGN_INSIDE, r);
    over_last = over_now;
  }
}

void InputBrowserTmp::hide_popup() {
  if (win && win->visible()) {
    fltk::exit_modal();
  }
}

void InputBrowserTmp::popup() {
  bool resize_only = false;

  if (!win || !win->visible()) {
    Group::current(0);

    if(!win) {
      win = new ComboWindow(0,0,0,0); // this will be moved later
      win->set_override();

      win->begin();
      list = new ComboBrowser(0,0,0,0);

	/** init style menu */
if(color_mw){ list->color(color_mw); }

if(box_mw){ list->box(box_mw); }
else{ list->box(UP_BOX); }

if(text_color_mw){ list->textcolor(text_color_mw); }

if(selection_color_mw){ list->selection_color(selection_color_mw); }

if(selection_textcolor_mw){ list->selection_textcolor(selection_textcolor_mw); }

if(textsize_mw){ list->textsize(textsize_mw); }

if(font_mw){ list->textfont(font_mw); }

	/** init style scrollbars menu */
if(s_box){
list->scrollbar.box(s_box);
list->hscrollbar.box(s_box); }

if(s_bbox){
list->scrollbar.buttonbox(s_bbox);
list->hscrollbar.buttonbox(s_bbox); }

if(s_color){
list->scrollbar.color(s_color);
list->hscrollbar.color(s_color); }

if(s_bcolor){
list->scrollbar.buttoncolor(s_bcolor);
list->hscrollbar.buttoncolor(s_bcolor); }

if(hig_color){
list->scrollbar.highlight_color(hig_color);
list->hscrollbar.highlight_color(hig_color); }


      list->callback(ComboBrowser::browser_cb, this);
      list->when(WHEN_CHANGED | WHEN_RELEASE_ALWAYS | WHEN_ENTER_KEY_ALWAYS);
      list->end();

      win->end();
      win->box(UP_BOX);

      mw = win;
      browser = list;
      ib = this;
    }

    share_list.other = this;
    list->list(&share_list);

    list->indented((type()&INDENTED) != 0);
    win->color(list->color());

  } else {
    resize_only = true;
  }

  list->layout();

  int W = list->width(); //+list->scrollbar.w();
  // magic constant 4 = border width/height (is there a way to calculate it?)
  int H = list->height() + 4;

  if (W > maxw_) W = maxw_;
  if (H > maxh_) H = maxh_;
  if (W < minw_) W = minw_;
  if (H < minh_) H = minh_;
  int X = event_x_root()-event_x();
  int Y = event_y_root()-event_y()+h();

  // I don't know what this code does, but it doesn't work
  // WAS: I believe it is trying to make the menu go above the combobox
  //      if it does not fit below on the screen
  /*      const Monitor& monitor = Monitor::find(event_x_root(), event_y_root());
	  int down = monitor.h() - Y;
	  int up = event_y_root() - event_y();
	  if (H > down) {
	  if (up > down) {
	  Y = event_y_root() - event_y() - H;
	  if (Y < 0) { Y = 0; H = up; }
	  } else {
	  H = down;
	  }
	  }
	  if (X + W > monitor.r()) {
	  X = monitor.r() - W;
	  if (X < 0) { X = 0; W = monitor.r(); }
	  }*/

  win->resize(X, Y, W, H);
  list->Widget::resize(W, H);

  // find the currently selected item in the list
  list->value(0);
  for (int i=0; i<list->children(); i++) {
    Widget* w=list->child(i);
    if (!strncmp(text(), w->label(), size())) {
      list->value(i);
      list->make_item_visible();
      break;
    }
  }

  if(resize_only) return;

  set_flag(PUSHED);
  redraw(DAMAGE_VALUE);

  win->exec(0, true);

  if(type()&NONEDITABLE) throw_focus();
  else fltk::focus(m_input);

  clear_flag(PUSHED);
  redraw(DAMAGE_VALUE);
}

Widget* InputBrowserTmp::item() const {
  if (list) return list->item(); else return 0;
}

Widget* InputBrowserTmp::item(Widget* v) const {
  if (list) return list->item(v); else return 0;
}

//
// End of "$Id: InputBrowserTmp.cxx The Dec 27 2006 Dmitry Perov  $" .
//
